
# 项目：Excel写入模块封装
# 模块：xlsxwriter的封装
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-10-12 08:24

from orange import *
from xlsxwriter import *
from xlsxwriter.format import Format
from xlsxwriter.worksheet import *

Pattern = R/r'([A-Z]{1,2})(\d*)([:_]([A-Z]{1,2})(\d*))?'
Row = R/r'(\{([+-]?\d+)\})'

DefaultFormat = (('currency', {'num_format': '#,##0.00'}),
                 ('rate', {'num_format': '0.0000%'}),
                 ('title', {'font_name': '黑体', 'font_size': 16,
                            'align': 'center'}),
                 ('h2', {'font_name': '黑体', 'font_size': 12,
                         'align': 'center'}),
                 ('mh2', {'font_name': '黑体', 'font_size': 12, 'text_wrap': True,
                          'align': 'center', 'valign': 'vcenter'}),
                 ('percent', {'num_format': '0.00%'}),
                 ('date', {'num_format': 'yyyy-mm-dd'}),
                 ('normal', {'text_wrap': True, 'valign': 'vcenter'}),
                 ('time', {'num_format': 'hh:mm:ss'}),
                 ('number', {'num_format': '#,##0'}),
                 ('header', {'font_name': '黑体',
                             'font_size': 12, 'align': 'center'}),
                 ('datetime', {'num_format': 'yyyy-mm-dd hh:mm:ss'}),
                 ('timestamp', {'num_format': 'yyyy-mm-dd hh:mm:ss.0'}))


class Book(Workbook):
    ''' 对Xlsxwriter模块进一步进行封装'''

    def __init__(self, filename=None, **kw):
        filename = str(Path(filename))
        super().__init__(filename, **kw)
        self._worksheet = None    # 设置当前的工作表为空
        self._worksheets = {}     # 设置当前的工作表清单为空
        self._formats = {}
        for name, val in DefaultFormat:
            self.add_format(val, name)

    def add_format(self, properties, name=None):
        _format = super().add_format(properties)
        if name:
            self._formats[name] = _format
            _format.name = name
            _format.properties = properties
        return _format

    def add_formats(self, properties):
        for name, property in properties.items():
            self.add_format(property, name)

    def set_widths(self, widths):
        [self.set_columns(col, width=width)for col, width in widths.items()]

    def set_columns(self, *columns, width=None, cell_format=None, options=None):
        ''' 设置当前工作表的列属性，允许同时设置多个，使用方法如下：
        book.set_columns('A:C','E:D','G:H',width=12)
        '''
        options = options or {}
        for column in columns:
            self.worksheet.set_column(column, width, cell_format, options)

    def newline(self):
        '''换行'''
        return self+1
    @property
    def worksheet(self):
        '''当前工作表'''
        return self._worksheet

    @worksheet.setter
    def worksheet(self, name):
        '''切换当前工作表'''
        worksheet = self._worksheets.get(name, None)
        if not worksheet:
            worksheet = self.add_worksheet(name)
            worksheet.row = 1
            self._worksheets[name] = worksheet
        self._worksheet = worksheet

    @convert_range_args
    def write(self, first_row, first_col, last_row=None, last_col=None,
              value=None, cell_format=None):
        if isinstance(cell_format, str):
            cell_format = self._formats.get(cell_format)
        if(last_row is None)or(first_row == last_row and
                               first_col == last_col):
            if isinstance(value, (tuple, list)):
                self.worksheet.write_row(first_row, first_col,
                                         value, cell_format)
            else:
                if isinstance(value, str) and value.startswith('='):
                    value = Row/value % self._convert   # 使用正则表达式替换
                self.worksheet.write(first_row, first_col, value,
                                     cell_format)
        else:
            self.worksheet.merge_range(first_row, first_col, last_row,
                                       last_col, value, cell_format)

    def _write(self, range, value, format=None):
        '''写入单元格，这个函数将被放弃，由write替代'''
        if isinstance(format, str):  # 格式代码检查
            format = self._formats.get(format)
        if ':' in range and not isinstance(value, (dict, tuple, list)):
            self.worksheet.merge_range(range, value, format)
        if ':' not in range:
            if isinstance(value, (list, tuple)):
                self.worksheet.write_row(range, value, format)
            else:
                if isinstance(value, str) and value.startswith('='):
                    value = Row/value % self._convert   # 使用正则表达式替换
                self.worksheet.write(range, value, format)

    def _convert(self, match):
        return str(int(match.groups()[1])+self.row)

    def __add__(self, val):
        '''向前移动当前行'''
        self.worksheet.row += val
        return self

    def __sub__(self, val):
        '''向后移动当前行'''
        self.worksheet.row -= val
        return self

    @property
    def row(self):
        '''获取当前工作表的行'''
        return self.worksheet.row

    @row.setter
    def row(self, currow):
        '''设置当前工作表的行'''
        self.worksheet.row = currow

    def __setitem__(self, name, val):
        '''向指定行写入数据'''
        if not isinstance(val, tuple):
            val = (val,)
        if isinstance(name, str):
            match = Pattern.fullmatch(name)
            if match:
                r = list(match.groups())
                if not r[1]:
                    r[1] = str(self.row)
                rg = ''.join(r[:2])
                if r[3]:
                    if not r[4]:
                        r[4] = str(self.row)
                    rg = '%s:%s%s' % (rg, r[3], r[4])
                self.write(rg, *val)
            else:
                raise Exception('单元格格式不正确')
        elif isinstance(name, int):
            self.write(self.row-1, name, None, None, *val)
        elif isinstance(name, tuple):
            args = list(name)
            args.extend([None]*4)
            self.write(*args[:4], *val)

    def __setattr__(self, name, val):
        '''向指定行写入数据'''
        if Pattern == name:
            self[name] = val
        else:
            super().__setattr__(name, val)

    def iter_rows(self, *datas, step=1):
        '''按行写入数据'''
        for d in zip(*datas):
            yield d
            self += step

    @property
    def table(self):
        return self.worksheet.table

    @convert_range_args
    def set_border(self, first_row, first_col, last_row, last_col,
                   left=None, right=None, bottom=None, top=None, border=2,
                   inner=1):
        self.worksheet._check_dimensions(first_row, first_col)
        self.worksheet._check_dimensions(last_row, last_col)
        if border:
            left = left or border
            right = right or border
            bottom = bottom or border
            top = top or border
        table = self.table
        default_format = {"left": inner, "right": inner, "top": inner,
                          "bottom": inner}

        def _replace(r, c, **kw):
            row = table[r]
            cell = row.get(c, cell_blank_tuple(None))
            fmt = cell.format
            kw['top'] = top if r == first_row else inner
            kw['bottom'] = bottom if r == last_row else inner
            kw['left'] = left if c == first_col else inner
            kw['right'] = right if c == last_col else inner
            name = ''.join([str(kw[name]) for name in
                            'left top right bottom'.split()])
            if fmt and hasattr(fmt, 'name'):
                name = name+'-'+fmt.name
            if not name in self._formats:
                if fmt and hasattr(fmt, 'properties'):
                    a = fmt.properties.copy()
                    a.update(kw)
                else:
                    a = kw
                new_fmt = self.add_format(a, name)
            else:
                new_fmt = self._formats.get(name)
            row[c] = cell._replace(format=new_fmt)
        [_replace(r, c)for r in range(first_row, last_row+1)
         for c in range(first_col, last_col+1)]

    @convert_range_args
    def add_table(self, first_row, first_col, last_row, last_col,
                  sheet=None, worksheet=None, header_format='header',
                  data=None, **kwargs):
        '''添加图表，如sheet为空，则使用默认的工作表'''
        worksheet = sheet or worksheet
        if worksheet:
            self.worksheet = worksheet
        elif not self.worksheet:
            raise Exception('当前 worksheet 未设置')
        columns = kwargs.get('columns')
        if columns:
            new_columns = []
            for idx, column in enumerate(columns):
                if 'width' in column:
                    self.set_columns("{0}:{0}".format(
                        xl_col_to_name(idx+first_col)),
                        width=column.get('width'))
                if 'format' in column or 'header_format' in column:
                    new_column = column.copy()
                    format = column.get("format")
                    if format and isinstance(format, str):
                        new_column['format'] = self._formats.get(format)
                    hformat = column.get("header_format")
                    if hformat and isinstance(hformat, str):
                        new_column['header_format'] = self._formats.get(
                            hformat)
                    new_columns.append(new_column)
                else:
                    new_columns.append(column)
            if header_format:
                if isinstance(header_format, str):
                    header_format = self._formats.get(header_format)
                for column in new_columns:
                    column['header_format'] = header_format
            kwargs['columns'] = new_columns
            last_col = first_col+len(columns)-1
        if data:
            if isinstance(data, generator):
                data = list(data)
            last_row = first_row+len(data)
            if kwargs.get('total_row', False):
                last_row += 1
            kwargs['data'] = data
        self.worksheet.add_table(first_row, first_col,
                                 last_row, last_col, kwargs)


if __name__ == '__main__':
    with Book('test.xlsx') as book:
        book.worksheet = 'test1'
        columns = [{'header': 'title', 'format': 'currency', 'width': 12},
                   {'header': 'name', 'format': 'percent'}]
        data = [('a', 'b'), (1, 2)]
        book.add_table('A1', data=data, columns=columns)



# 项目：标准库函数
# 模块：协程模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-12-26 13:34

'''
对asyncio库进行进一步封装，主要功能如下：

1. 执行一个或并行运行多个协程：
    start(coro1,coro2,...)
2. 中断协程，并等待另一个协程的运行结果：
    result=await coro()
3. 在协程中，并发多个协程：
    await wait(*coros)
'''
import asyncio

__all__ = 'start', 'wait', 'wait_for', 'sleep', 'iscoroutine', 'batch', 'run'

wait = asyncio.wait
wait_for = asyncio.wait_for
sleep = asyncio.sleep
iscoroutine = asyncio.iscoroutine


def batch(coro, *datas):
    # 使用一个协程函数批量生成协程
    return wait(map(coro, *datas))


def start(*coros):
    # 执行协程，如coro为多个，则全部并发执行。
    try:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        coro = coros[0] if len(coros) == 1 else wait(coros)
        loop.run_until_complete(coro)
    finally:
        loop.close()


def run(*coros):
    loop = asyncio.get_event_loop()
    coro = coros[0] if len(coros) == 1 else wait(coros)
    loop.run_until_complete(coro)



# 项目：Python开发工具包
# 模块：工具包
# 作者：黄涛
# 创建：2015-9-2
# 修订：2018-2-2 新增 __version__

from .path import *
from .version import *
from .deploy import *
from .debug import *
from .htutil import *
from .dateutil import *
from .regex import *
from .mail import *
from .click import *
from .__version__ import version as __version__

__all__ = 'get_ver', 'Path', 'get_path',\
    'first', 'last', 'Ver', 'decode',\
    'setup', 'decorator', 'trace', 'config_log', 'ensure', 'info',\
    'classproperty', 'is_installed', 'is_dev', 'cachedproperty',\
    'read_shell', 'write_shell', 'exec_shell', 'wlen',\
    'encrypt', 'decrypt', 'get_py', 'split', 'deprecation',\
    'LOCAL', 'UTC', 'now', 'datetime', 'fprint', 'date_add', 'ONEDAY', 'LTZ',\
    'ONESECOND', 'R', 'sendmail', 'tsendmail', 'Mail', 'PY', 'MailClient',\
    'convert_cls_name', 'verbose', 'arg', 'command', 'generator','__version__'




# 项目：标准库函数
# 模块：配置mongodb数据库
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-11-19 10:18

import sys
import platform
from orange.deploy import *
from orange import exec_shell, read_shell
from orange.pyver import Ver, get_cur_ver

MONGOCONFIG = '''
systemLog:
  destination: file
  path: "{logpath}/mongodb.log"
  logAppend: true
storage:
  dbPath: "{dbpath}"
  directoryPerDB: true
  engine: "{engine}"
net:
  bindIp: 127.0.0.1
  port: 27017
'''
MONGOFILES = ('mongo.exe', 'mongod.exe', 'mongodump.exe',
              'mongoexport.exe', 'mongoimport.exe')

SERVERNAME = 'MongoDb'


def win_deploy():
    print('停止 MongoDb 服务')
    exec_shell('sc stop %s' % (SERVERNAME))
    cur_prg_path = get_cur_ver(Path('%PROGRAMFILES%/MongoDB').rglob('bin'))
    print('最新版程序安装路径：%s' % (cur_prg_path))
    dest = Path('%windir%')
    for exefile in MONGOFILES:
        dexefile = dest/exefile
        if dexefile.exists():
            dexefile.unlink()
        dexefile.symlink_to(cur_prg_path/exefile)
        print('连接 %s 到 %s 成功' % (dexefile, cur_prg_path/exefile))

    root = get_path(SERVERNAME, False)[0]
    data_path = root / 'data'
    if not data_path.exists():
        data_path.ensure()
        print('创建数据目录：%s' % (data_path))
    config_file = root/'mongo.conf'
    print('写入配置文件 %s ' % (config_file))
    config = {
        'dbpath': data_path.as_posix(),
        'logpath': root.as_posix(),
        'engine': 'wiredTiger'}
    if platform.architecture()[0] != '64bit':
        config['engine'] = 'mmapv1'
        print('本机使用32位处理器，使用 mmapv1 引擎')
    config_file.text = MONGOCONFIG.format(**config)

    print('删除服务配置')
    exec_shell('sc delete %s' % (SERVERNAME))
    print('重新配置服务')
    cmd = '%s --install --serviceName "%s" --config "%s"' % (
        'mongod.exe', SERVERNAME, config_file)
    print(cmd)
    exec_shell(cmd)
    print('启动 MongoDB 服务')
    exec_shell('sc start %s' % (SERVERNAME))
    input('Press any key to continue')


PLIST_CONTENT='''<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
  <dict>
    <key>KeepAlive</key>
    <true/>
    <key>Label</key>
    <string>com.mongodb.mongod</string>
    <key>ProgramArguments</key>
    <array>
        <string>/usr/local/bin/mongod</string>
        <string>--config</string>
        <string>/usr/local/etc/mongod.conf</string>
    </array>
  </dict>
</plist>
'''

def darwin_deploy():
    config = {
        'dbpath': '/usr/local/var/mongodb',
        'logpath': 'usr/local/var/log',
        'engine': 'wiredTiger'}
    config_path=Path('/usr/local/etc/mongod.conf')
    plist_file=Path('/Library/LaunchDaemons/com.mongodb.plist')
    config_path.text=MONGOCONFIG.format(**config)
    plist_file.text=PLIST_CONTENT


def main():
    import sys
    if sys.platform == 'win32':
        win_deploy()
    elif sys.platform == 'darwin':
        darwin_deploy()
    else:
        print('操作系统%下的配置未实现')

if __name__ == '__main__':
    main()



# 项目：标准库函数模块
# 模块：配置程序
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2015-05-26 16:12
# 修订：2016-10-28 增加

import os
import sys
import sysconfig
import orange
import atexit
from configparser import ConfigParser
from orange import decrypt, encrypt, Path


class Config:
    def __init__(self, config_path=None, data_path=None,
                 is_dev=None, project=None):
        if is_dev is None:
            self.is_dev = orange.is_dev()
        else:
            self.is_dev = is_dev
        if project is None:
            self.project = os.path.splitext(
                os.path.basename(sys.argv[0]))[0]
        else:
            self.project = project
        self.os_name = os.name
        if self.os_name == 'posix':
            config_ext = '.conf'
            self.config_path = os.path.expanduser(
                "~/.%s" % (self.project))
        else:
            config_ext = '.ini'
            self.config_path = os.path.join(os.getenv("APPDATA"),
                                            self.project)
        if self.is_dev:
            self.config_path = os.path.abspath("appdata")
        else:
            if config_path:
                self.config_path = config_path
        if not hasattr(self, "data_path"):
            self.data_path = self.config_path
        self.config_file = os.path.join(self.data_path,
                                        self.project+config_ext)
        self.modified = False
        self.load_config()

    def load_config(self, files=None):
        if files is None:
            files = self.config_file
        if not hasattr(self, 'parser'):
            self.parser = ConfigParser()
        self.parser.clear()
        self.parser.read(files, encoding='utf8')
        self.modified = False
        if isinstance(files, str):
            self.cur_file = files
        else:
            self.cur_file = files[-1]

    def __setitem__(self, name, data):
        return self.update(name, data)

    @property
    def sections(self):
        return self.parser.sections()

    def get(self, section):
        if self.parser.has_section(section):
            d = {}
            for option, value in self.parser.items(section):
                if self.is_passwd(option):
                    value = decrypt(value)
                d[option] = value
            return d

    def update(self, section, data):
        self.modified = True
        if not self.parser.has_section(section):
            self.parser.add_section(section)
        for option, value in data.items():
            if self.is_passwd(option):
                value = encrypt(value)
            self.parser.set(section, option, str(value))

    def get_many(self, *sections):
        return dict([(section, self.get(section)) for section in sections])

    def update_many(self, datas):
        for section, data in datas.items():
            self.update(section, data)

    def is_passwd(self, key):
        return key.lower() in ('passwd', 'password')

    def save_config(self):
        # ensure_path(os.path.dirname(self.cur_file))
        Path(self.cur_file).parent.ensure()
        if self.modified:
            with open(self.cur_file, 'w', encoding='utf8')as fn:
                self.parser.write(fn)

    def init_logging(self):
        import logging
        file_name = os.path.join(self.data_path,
                                 self.project+'.log')
        if self.is_dev:
            level = 'DEBUG'
        else:
            level = 'WARN'
        default = {
            'filename': file_name,
            'level': level,
            'format': '%(asctime)s %(levelname)s\t%(message)s',
            'datefmt': '%Y-%m-%d %H:%M'}
        default.update(self.get('logging'))
        logging.basicConfig(**default)


_config = None


def config(*args, **kw):
    global _config
    if _config is None:
        _config = Config(*args, **kw)
    return _config


@atexit.register
def save_config():
    if _config and _config.modified:
        _config.save_config()



#!/usr/bin/env python3
# 项目：克隆github.com上的项目
# 模块：命令行模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2015-06-11 12:28
# 修订：2016-11-18 采用Parser来分析参数

#from stdlib import parse_args,exec_shell
from orange import arg, exec_shell
import sys


@arg('repos', nargs='+', metavar='repo', help="要下载的软件仓库，可以为多个")
@arg('-u', '--user', nargs='?', help='要下载的用户名,默认为本人的仓库')
def proc(repos=None, user=None, protocol='SSH'):
    if user is None:
        from configparser import ConfigParser
        import os
        config = ConfigParser()
        config.read([os.path.expanduser('~/.gitconfig')])
        try:
            user = config.get('user', 'name')
        except:
            raise Exception('用户不存在！')
    protocol = protocol.upper()
    URL = 'git@github.com:%s' % (user) if protocol == 'SSH' else \
        'https://github.com/%s' % (user)
    for repo in repos:
        url = '%s/%s.git' % (URL, repo)
        print('cloning', url)
        exec_shell('git clone %s' % (url))


if __name__ == '__main__':
    proc()



#!/usr/bin/env python3
from orange import setup

console_scripts = ['conv=orange.path:convert',
                   #'pytest=orange.pytools:pytest',
                   'pysdist=orange.pytools:pysdist',
                   'pyupload=orange.pytools:pyupload',
                   'canshu=orange.ggcs:canshu',
                   #'mongodb=orange.mongodb:main',
                   'pyver=orange.pyver:VersionMgr.main',
                   'plist=orange.plist:main',
                   'pyinit=orange.init:main',
                   'gclone=orange.gclone:proc',
                   'mongodeploy=orange.mongodb:main',
                   'fkgfw=orange.fkgfw:main',
                   'sysinit=orange.sysinit:main']
setup(
    name='orange-kit',
    platforms='any',
    description='orange',
    long_description='orange',
    url='https://github.com/huangtao-sh/orange.git',
    cscripts=console_scripts,
    # entry_points={'console_scripts':console_scripts},
        license='GPL',
)
